
// $Id: ChangeListener.java,v 1.3 1999/03/22 09:26:49 larsga Exp $

package XSA;

/**
 * This interface must be implemented by classes that wish to listen to
 * XSA change notifications. Currently only used by DefaultClient.
 */

public interface ChangeListener {

  /**
   * Called whenever an error is detected.
   * @param url The URL that was being checked when the error was detected.
   * @param msg The error message.
   */

  public void error(String url, String msg);
  
  /**
   * Called to inform the listener that a vendor has changed name.
   */
  
  public void vendorNameChanged(Vendor oldVendor, String newName);

  /**
   * Called to inform the listener that a vendor has changed email address.
   */

  public void vendorEmailChanged(Vendor oldVendor, String newEmail);  

  /**
   * Called to inform the listener that a vendor has changed home page address.
   */

  public void vendorURLChanged(Vendor oldVendor, String newURL);  

  /**
   * Called to inform the listener that a vendor has a new product.
   */

  public void vendorNewProduct(Vendor vendor, Product product);

  /**
   * Called to inform the listener that a vendor XSA document has moved.
   */

  public void vendorDocumentMoved(Vendor vendor, String newURL);
  
  /**
   * Called to inform the listener that a vendor no longer seems to offer
   * a product.
   */

  public void vendorProductMissing(Vendor vendor, Product product);

  /**
   * Called to inform the listener that a product has changed name.
   */

  public void productNameChanged(Vendor oldvendor, Product oldprod,
				 String newName);

  /**
   * Called to inform the listener that a new version of a product has
   * been announced.
   */

  public void productNewVersion(Vendor oldvendor, Product oldprod,
				Product newprod);

  /**
   * Called to informn the listener that a product information URL has
   * changed.
   */

  public void productURLChanged(Vendor oldvendor, Product oldprod,
				String newURL);

  /**
   * Called to inform the listener that checking is now complete.
   */

  public void done();
}
