
// $Id: DefaultClient.java,v 1.4 1999/03/22 09:26:49 larsga Exp $

package XSA;

import java.io.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * This class is intended to be the XSA client used by most applications. It
 * compares the results of an XSA check with the results of the previous
 * check. The results of the previous check are stored using Java
 * serialization.
 */

public class DefaultClient implements XSAClient {
  private Hashtable      oldlist;  // url -> vendor
  private Hashtable      newlist;  // url -> vendor
  private String         savefile;
  private ChangeListener listener;

  public DefaultClient() {
    savefile="xsadefcl.sav";
    listener=new SilentListener();
  }

  /**
   * Tells the client which result save file to use. (This is used both for
   * reading and saving results.)
   */
  
  public void setSavefile(String filename) {
    savefile=filename;
  }

  /**
   * Tells the client which listener to notify for changes since the last
   * check.
   */

  public void setChangeListener(ChangeListener list) {
    listener=list;
  }
  
  // --- XSAClient methods

  public void checkingStarts() {
    // Load results of last check
    oldlist=(Hashtable) read(savefile);
    newlist=new Hashtable();
    
    // Do existence check and exit
    if (oldlist==null) 
      oldlist=new Hashtable();
  }

  public void checkingDone() {
    // Store results of this check
    write(savefile,newlist);
    listener.done();
  }

  public synchronized void error(String url, String msg) {
    listener.error(url,msg);
  }

  public synchronized void checkedVendor(Vendor vendor) {
    Vendor oldvendor=(Vendor) oldlist.get(vendor.getURL());
    newlist.put(vendor.getURL(),vendor);
    if (oldvendor==null) return;

    if (!oldvendor.getName().equals(vendor.getName()))
      listener.vendorNameChanged(oldvendor,vendor.getName());

    if (!oldvendor.getEmail().equals(vendor.getEmail()))
      listener.vendorEmailChanged(oldvendor,vendor.getEmail());

    if (!oldvendor.getHomepage().equals(vendor.getHomepage()))
      listener.vendorURLChanged(oldvendor,vendor.getHomepage());

    // Compare products
    Hashtable oldproducts=new Hashtable();
    Enumeration prods=oldvendor.getProducts();
    Product oldprod,newprod;

    while (prods.hasMoreElements()) {
      oldprod=(Product) prods.nextElement();
      oldproducts.put(oldprod.getId(),oldprod);
    }

    prods=vendor.getProducts();
    while (prods.hasMoreElements()) {
      newprod=(Product) prods.nextElement();
      oldprod=(Product) oldproducts.get(newprod.getId());

//       System.out.println("New: "+newprod.getName());
//       System.out.println("Ver: "+newprod.getVersion());
//       if (oldprod!=null) {
// 	System.out.println("Old: "+oldprod.getName());
// 	System.out.println("Ver: "+oldprod.getVersion());
//       }
      
      if (oldprod==null)
	listener.vendorNewProduct(oldvendor,newprod);
      else if (!oldprod.getName().equals(newprod.getName()))
	listener.productNameChanged(oldvendor,oldprod,newprod.getName());
      else if (!oldprod.getVersion().equals(newprod.getVersion())) {
	listener.productNewVersion(oldvendor,oldprod,newprod);
      } else if (!oldprod.getInfoURL().equals(newprod.getInfoURL()))
	listener.productURLChanged(oldvendor,oldprod,newprod.getInfoURL());

      if (oldprod!=null) oldproducts.remove(oldprod.getId());
    }

    prods=oldproducts.elements();
    while (prods.hasMoreElements()) {
      oldprod=(Product) prods.nextElement();
      listener.vendorProductMissing(oldvendor,oldprod);
    }
  }

  public void documentMoved(String oldUrl, String newUrl) {
    Vendor vendor=(Vendor) oldlist.get(oldUrl);
    newlist.put(oldUrl,vendor);
    oldlist.put(newUrl,vendor);
    listener.vendorDocumentMoved(vendor,newUrl);    
  }

  // --- Internal methods

  private Object read(String filename) {
    Object obj;

    try {
      FileInputStream istream = new FileInputStream(filename);
      ObjectInputStream p = new ObjectInputStream(istream);

      obj = p.readObject();

      istream.close();
    }
    catch (FileNotFoundException e) {
      obj = null;
    }
    catch (Exception e) {
      obj = null;
      error(null,"Loading old results: "+e.toString());
    }      

    return obj;
  }  

  private void write(String filename, Object obj) {
    try {
      FileOutputStream ostream = new FileOutputStream(filename);
      ObjectOutputStream p = new ObjectOutputStream(ostream);
      p.writeObject(obj);
      p.flush();
      ostream.close();
    }
    catch (IOException ex) {
      error(null,"Storing results: "+ex.toString());
    }
  }  
}
