
package XSA;

import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * This class represents and encapsulates information about software
 * products and can be queried for information.
 */

public class Product implements Serializable {
  private String name;
  private String id;
  private String version;
  private Date lastRelease;
  private String infoURL;
  private String changes;

  public Product() {
    name="";
    id="";
    version="";
    infoURL="";
  }
  
  /**
   * Returns the product name.
   */

  public String getName() {
    return name;
  }

  /**
   * Returns the ID of the product in the XSA document.
   */

  public String getId() {
    return id;
  }
  
  /**
   * Returns the current product version.
   */

  public String getVersion() {
    return version;
  }

  /**
   * Returns the date when the product was last released.
   */

  public Date getLastRelease() {
    return lastRelease;
  }

  /**
   * Returns the URL to the product home page.
   */

  public String getInfoURL() {
    return infoURL;
  }

  /**
   * Returns a textual description of the changes since the last release of
   * the product.
   * @return Will return null if the changes element was not present.
   */

  public String getChanges() {
    return changes;
  }

  // --- Internal setting methods ---

  /**
   * Sets the product name.
   */

  protected void setName(String _name) {
    name=_name;
  }

  /**
   * Sets the product id.
   */

  protected void setId(String _id) {
    id=_id;
  }
  
  /**
   * Sets the current product version.
   */

  protected void setVersion(String _version) {
    version=_version;
  }

  /**
   * Sets the date when the product was last released.
   * @param _lastRelease Must be in YYYYMMDD format.
   * @exception XSAInvalidValueException Thrown if the date value is not
   *                                     accepted.
   */

  protected void setLastRelease(String _lastRelease)
    throws XSAInvalidValueException {
    try {
      Calendar cal=new GregorianCalendar();
      cal.setLenient(false);
      cal.set(Integer.parseInt(_lastRelease.substring(0,4)),
	      Integer.parseInt(_lastRelease.substring(4,6))-1,
	      Integer.parseInt(_lastRelease.substring(6,8)));
      
      lastRelease=cal.getTime();
//       DateFormat df=new SimpleDateFormat("dd.MMM.yyyy");
//       System.out.println(df.format(lastRelease));
    }
    catch (Exception e) {
      throw new XSAInvalidValueException("'"+_lastRelease+
					 "' is not a valid date.");
    }
  }

  /**
   * Sets the URL to the product home page.
   */

  protected void setInfoURL(String _infoURL) {
    infoURL=_infoURL;
  }

  /**
   * Sets the textual description of the changes since the last release of
   * the product.
   */

  protected void setChanges(String _changes) {
    changes=_changes;
  }
  
}
