
// $Id: SAXHandler.java,v 1.3 1999/03/22 09:26:53 larsga Exp $

package XSA;

import java.util.Hashtable;

import org.xml.sax.*;

/**  
 * An abstract SAX DocumentHandler for interpreting XML documents and
 * building objects ready to be returned to the XSA client. 
 */

abstract class SAXHandler extends HandlerBase implements XSADocumentHandler {
  // Some protected constants
  protected final int NOWHERE=0;
  protected final int IN_VENDOR=1;
  protected final int IN_PRODUCT=2;

  protected final char LF="\r".charAt(0);
  protected final char CR="\n".charAt(0);
  protected final char SPC=" ".charAt(0);
  protected final char TAB="\t".charAt(0);

  // Variables used to store gathered information
  protected Vendor       vendor;
  protected Product      product;
  protected StringBuffer contents;

  // Some protected variables for tracking state
  protected int          context;
  protected String       element;
  
  // Other variables
  protected Locator      loc;
  protected Hashtable    pids;
  protected ObjectFactory factory;
  protected XSAClient     client;

  /**
   * Creates a SAXHandler, setting it up for use.
   */

  public SAXHandler(XSAClient _client, String url, Hashtable _pids,
		    ObjectFactory _factory) {
    context=NOWHERE;
    element=null;
    factory=_factory;
    client=_client;
    vendor=factory.createVendor(url);
    pids=_pids;
  }

  // ===== External interface =====

  public Vendor getVendor() {
    return vendor;
  }
  
  // ===== SAX event methods =====
  
  public void setDocumentLocator (Locator locator) {
    loc=locator;
  }
  
  public void characters (char ch[], int start, int length) 
    throws SAXException {
    if (element!=null) 
      contents.append(ch,start,length);
  }

  // ===== Internal whitespace treatment methods =====

  protected String removeWS(StringBuffer value) {
    char data[]=new char[value.length()];
    char ch;
    int pos=0;
    
    for (int ix=0; ix<value.length(); ix++) {
      ch=value.charAt(ix);
      if (! (ch==CR || ch==LF || ch==SPC || ch==TAB)) 
	data[pos++]=ch;
    }

    return new String(data,0,pos);
  }

  protected String normalizeWS(StringBuffer value) {
    char data[]=new char[value.length()];
    char ch;
    int pos=0;
    boolean prev_WS=false;
    
    for (int ix=0; ix<value.length(); ix++) {
      ch=value.charAt(ix);

      if (ch==LF || ch==CR || ch==SPC || ch==TAB) {
	if (!prev_WS) {
	  data[pos++]=SPC;
	  prev_WS=true;
	}
      } else {
	data[pos++]=ch;
	prev_WS=false;
      }
    }

    // Stripping WS before and after text
    
    int start=0, length=pos;
    if (data[0]==SPC) start=1;
    if (data[pos-1]==SPC) length--;
    
    return new String(data,start,length);
  }

}
