
package XSA.Util;

// $Id: AnySAXParserFactory.java,v 1.3 1999/03/22 09:27:17 larsga Exp $

import org.xml.sax.Parser;
import org.xml.sax.helpers.ParserFactory;

/**
 * This parser factory has a preset list of known validating SAX parsers
 * which it runs through to see if any of them are installed. If the
 * org.xml.sax.parser property is set this parser will be used.
 * @author Lars Marius Garshol (larsga@ifi.uio.no)
 */

public class AnySAXParserFactory {
  private static String[] allDrivers=  
    {// Trying the non-validating parsers first
     "com.jclark.xml.sax.Driver",
     "com.microstar.xml.SAXDriver",
     "com.megginson.sax.LarkDriver",
     "com.sun.xml.parser.Parser",

     // Then going on to the validating ones
     "com.sun.xml.parser.ValidatingParser",
     "com.megginson.sax.LarvalDriver",
     "com.ibm.xml.parser.SAXDriver",
     "fr.loria.xml.sax.SAXDriver",
     "org.openxml.sax.SAXParser"
    };

  /**
   * Returns the name of an installed validating parser by first trying the
   * org.xml.sax.parser property and a preset list of parsers.
   * @returns The class name of the driver or null if none was found.
   */

  public static String findParser() {
    return AnySAXParserFactory.findParser(allDrivers);
  }
  
  /**
   * Returns the name of an installed validating parser by first trying the
   * org.xml.sax.parser property and the supplied list of parsers.
   * @returns The class name of the driver or null if none was found.
   */
  
  public static String findParser(String[] drivers) {
    // Trying org.xml.sax.parser first
    try {   		
      ParserFactory.makeParser();
      return System.getProperty("org.xml.sax.parser"); 
    }
    catch (Exception e) {} // We failed for some reason. Now try the others.

    // Trying the preset list
    for (int ix=0; ix<drivers.length; ix++) {
      try {
	ParserFactory.makeParser(drivers[ix]);
	return drivers[ix];
      }
      catch (Exception e) { } // Failed. Try another.
    }

    return null; // Didn't find any.
  }
  
}
