
package XSA;

import java.io.Serializable;
import java.util.Vector;
import java.util.Enumeration;

/**
 * This class represents a software vendor and encapsulates information
 * about said vendor.
 */

public class Vendor implements Serializable {
  private String name;
  private String email;
  private String homepage;
  private String docURL;
  private Vector products;
  
  /**
   * Creates an empty vendor object.
   */

  public Vendor(String url) {
    products=new Vector();
    docURL=url;

    name="";
    email="";
    homepage="";
  }
  
  /**
   * Returns the name of the vendor.
   */

  public String getName() {
    return name;
  }

  /**
   * Returns the email address of the vendor.
   */

  public String getEmail() {
    return email;
  }

  /**
   * Returns the URL of the vendors home page.
   */

  public String getHomepage() {
    return homepage;
  }

  /**
   * Returns the URL of the vendors XSA document.
   */

  public String getURL() {
    return docURL;
  }

  /**
   * Returns the products registered on this vendor.
   * @return An enumeration of Product objects. If a hashtable of IDs was given
   *         to the engine, only products with those IDs will be in the
   *         Enumeration.
   * @see Product
   */

  public Enumeration getProducts() {
    return products.elements();
  }

  // --- Methods internal to the XSA package.

  /**
   * Sets the name of the vendor.
   */

  protected void setName(String _name) {
    name=_name;
  }

  /**
   * Sets the email address of the vendor.
   */

  protected void setEmail(String _email) {
    email=_email;
  }

  /**
   * Sets the URL of the vendors home page.
   */

  protected void setHomepage(String _homepage) {
    homepage=_homepage;
  }

  /**
   * Adds a product to this vendors list of products.
   */

  protected void addProduct(Product newProduct) {
    products.addElement(newProduct);
  }
  
}
