
// $Id: XSAClient.java,v 1.2 1999/03/22 09:26:55 larsga Exp $

package XSA;

/**
 * This interface must be implemented by all XSA clients.
 */

public interface XSAClient {

  /**
   * Notifies the client that XSA checking is about to begin. This method
   * is always called before error or checkedDocument are called.
   */

  public void checkingStarts();   
  
  /**
   * Notifies the client of an error that occurred while checking the
   * given URL. This method should be synchronized, in case the XSA engine
   * is multithreaded.
   * @param url The URL of the XSA document the engine was checking when the
   *            error occurred.
   * @param message A textual error message describing the error.      
   */
  
  public void error(String url, String message);

  /**
   * Notifies the client that an XSA document has been checked. This method
   * should be synchronized, in case the XSA engine is multithreaded.
   * @param vendor A Vendor object containing the information in the XSA
   *               document.
   * @see Vendor
   */

  public void checkedVendor(Vendor vendor);

  /**
   * Notifies the client that no more documents will be checked. This method
   * is always the last to be called.
   */

  public void checkingDone();

  /**
   * Notifies the client that an XSA document has been moved. This method
   * is called when the XSA engine receives an HTTP redirect when trying
   * to access a document.
   */

  public void documentMoved(String oldUrl, String newUrl);
}
