
// $Id: XSASAXHandler.java,v 1.3 1999/03/22 09:26:58 larsga Exp $

package XSA;

import java.util.Hashtable;

import org.xml.sax.*;

/**
 * A SAX DocumentHandler and ErrorHandler for interpreting XSA
 * documents and building objects ready to be returned to the XSA client.
 */

class XSASAXHandler extends SAXHandler {

  public XSASAXHandler(XSAClient client, String url, Hashtable _pids,
		       ObjectFactory _factory) {
    super(client, url,_pids,_factory);
  }

  // ===== SAX event methods =====
  
  public void startElement (String name, AttributeList attributes)
    throws SAXException {
    if (name.equals("vendor"))
      context=IN_VENDOR;
    else if (name.equals("product") &&
	      (pids==null || pids.containsKey(attributes.getValue("id")))) {
      product=factory.createProduct();
      product.setId(attributes.getValue("id"));
      vendor.addProduct(product);
      context=IN_PRODUCT;
    } else if (!name.equals("xsa") && element==null) {
      element=name;
      contents=new StringBuffer();
    }
  }

  public void endElement (String name) throws SAXException {
    if (name.equals("vendor") || name.equals("product")) {
      context=NOWHERE;
    } else if (name.equals(element)) {
      try {
	if (context==IN_VENDOR) {
	  if (element.equals("name"))
	    vendor.setName(normalizeWS(contents));
	  else if (element.equals("email"))
	    vendor.setEmail(removeWS(contents));
	  else if (element.equals("url"))
	    vendor.setHomepage(removeWS(contents));
	} else if (context==IN_PRODUCT) {
	  if (element.equals("name"))
	    product.setName(normalizeWS(contents));
	  else if (element.equals("version"))
	    product.setVersion(normalizeWS(contents));
	  else if (element.equals("last-release"))
	    product.setLastRelease(removeWS(contents));
	  else if (element.equals("info-url"))
	    product.setInfoURL(removeWS(contents));
	  else if (element.equals("changes"))
	    product.setChanges(contents.toString());
	}
      }
      catch (XSAInvalidValueException e) {
	if (loc!=null) {
	  String location="(Line: "+Integer.toString(loc.getLineNumber())+
  	                  " Column: "+Integer.toString(loc.getColumnNumber())+
	                  ")";
	
	  client.error(vendor.getURL(),e.getMessage()+" "+location);
	} else {
	  client.error(vendor.getURL(),e.getMessage());
	}
      }
      
      element=null;
    }
  }
}
